﻿using Microsoft.EntityFrameworkCore;
using PaydayBackend.Models;

namespace PaydayBackend.Services;

public interface IPublicService
{
    public Task<IEnumerable<UniversityDto>> GetAllUniversity();
    public Task<IEnumerable<UniversityDirectionDto>?> GetAllUniversityDirectionByUniversityId(long universityId);
    public Task<IEnumerable<LoanTerm>?> GetAllLoansByDirectionCost(long universityDirectionId);
    public Task<long> GetMinPlaceCostByUniversityId(long universityId);
}

public class PublicService : IPublicService
{
    private DatabaseContext _databaseContext;

    public PublicService(DatabaseContext databaseContext)
    {
        _databaseContext = databaseContext;
    }
    
    
    
    public async Task<IEnumerable<UniversityDto>> GetAllUniversity()
    {
        var result = await _databaseContext.Universities.ToListAsync();
        
        List<UniversityDto> dtos = new List<UniversityDto>();
        foreach (var item in result)
        {
            UniversityDto dto = new UniversityDto(item.Id, item.Name, item.FullName, item.ImageUrl);
            dto.MinPlaceCost = await GetMinPlaceCostByUniversityId(item.Id);
            dtos.Add(dto);
        }

        return dtos;
    }
    
    public async Task<IEnumerable<UniversityDirectionDto>?> GetAllUniversityDirectionByUniversityId(long universityId)
    {   
        if (await _databaseContext.Universities.Where(x => x.Id == universityId).FirstOrDefaultAsync() == null)
            return null;
        
        var result = await _databaseContext.UniversityDirections.Where(x => x.UniversityId == universityId).ToListAsync();
        List<UniversityDirectionDto> dtos = new List<UniversityDirectionDto>();
        foreach (var item in result)
        {
            UniversityDirectionDto dto = new UniversityDirectionDto(item.Id, item.UniversityId, item.Code, item.Name, item.Url, item.PlaceCost);
            dtos.Add(dto);
        }

        return dtos;
    }

    public async Task<IEnumerable<LoanTerm>?> GetAllLoansByDirectionCost(long universityDirectionId)
    {
        var universityDirection = await _databaseContext.UniversityDirections.Where(x => x.Id == universityDirectionId).FirstOrDefaultAsync();
        if (universityDirection == null)
            return null;
        
        List<LoanTerm> loanTerms = await _databaseContext.LoanTerms.Where(x => x.MaxCost >= universityDirection.PlaceCost).ToListAsync();
        return loanTerms;
    }
    
    public async Task<long> GetMinPlaceCostByUniversityId(long universityId)
    {
        try
        {
            var universityDirection = await _databaseContext.UniversityDirections.OrderByDescending(x => x.PlaceCost).Where(x => x.UniversityId == universityId).FirstAsync();
            if (universityDirection == null)
                return 0;
            return universityDirection.PlaceCost;
        }
        catch (Exception e)
        {
            return 0;
        }
    }
}